@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright (C) 2011 Igalia, S.L.
+# Copyright (C) 2011-2013 Igalia, S.L.
 #
 # Author: Joanmarie Diggs <jdiggs@igalia.com>
 #
@@ -24,7 +24,7 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (C) 2011 Igalia, S.L."
+__copyright__ = "Copyright (C) 2011-2013 Igalia, S.L."
 __license__   = "LGPL"
 
 import pyatspi
@@ -68,7 +68,7 @@ class LabelInference:
             result = self.inferFromTextLeft(obj)
             debug.println(debug.LEVEL_FINE, "INFER - Text Left: %s" % result)
         if not result or self._preferRight(obj):
-            result = self.inferFromTextRight(obj)
+            result = self.inferFromTextRight(obj) or result
             debug.println(debug.LEVEL_FINE, "INFER - Text Right: %s" % result)
         if not result:
             result = self.inferFromTable(obj)
@@ -110,17 +110,33 @@ class LabelInference:
         onRightRoles = [pyatspi.ROLE_CHECK_BOX, pyatspi.ROLE_RADIO_BUTTON]
         return obj.getRole() in onRightRoles
 
+    def _preventRight(self, obj):
+        """Returns True if we should not permit inference based on text to
+        the right for the object obj."""
+
+        roles = [pyatspi.ROLE_COMBO_BOX,
+                 pyatspi.ROLE_LIST,
+                 pyatspi.ROLE_LIST_BOX]
+
+        return obj.getRole() in roles
+
     def _preferTop(self, obj):
         """Returns True if we should prefer text above, rather than below for
         the object obj."""
 
-        roles = [pyatspi.ROLE_COMBO_BOX, pyatspi.ROLE_LIST]
+        roles = [pyatspi.ROLE_COMBO_BOX,
+                 pyatspi.ROLE_LIST,
+                 pyatspi.ROLE_LIST_BOX]
 
-        # Put new-to-pyatspi roles here.
-        try:
-            roles.append(pyatspi.ROLE_LIST_BOX)
-        except:
-            pass
+        return obj.getRole() in roles
+
+    def _preventBelow(self, obj):
+        """Returns True if we should not permit inference based on text below
+        the object obj."""
+
+        roles = [pyatspi.ROLE_COMBO_BOX,
+                 pyatspi.ROLE_LIST,
+                 pyatspi.ROLE_LIST_BOX]
 
         return obj.getRole() in roles
 
@@ -147,8 +163,8 @@ class LabelInference:
             return True
 
         string = text.getText(0, -1).strip()
-        if string.find(self._script.EMBEDDED_OBJECT_CHARACTER) > -1:
-            return len(string) == 1
+        if string.count(self._script.EMBEDDED_OBJECT_CHARACTER) > 1:
+            return False
 
         return True
 
@@ -203,6 +219,8 @@ class LabelInference:
         else:
             skipTextExtents = [pyatspi.ROLE_ENTRY, pyatspi.ROLE_PASSWORD_TEXT]
             if not obj.getRole() in skipTextExtents:
+                if endOffset == -1:
+                    endOffset = text.characterCount
                 extents = text.getRangeExtents(startOffset, endOffset, 0)
 
         if not (extents[2] and extents[3]):
@@ -227,8 +245,7 @@ class LabelInference:
             return ''
 
         strings = [content[3] or content[0].name for content in contents]
-        strings = [x.strip() for x in strings]
-        return ' '.join(strings)
+        return ''.join(strings)
 
     def _getLineContents(self, obj):
         """Get the (obj, startOffset, endOffset, string) tuples for the line
@@ -245,7 +262,23 @@ class LabelInference:
             obj = obj.parent
 
         boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-        rv = self._script.utilities.getObjectsFromEOCs(obj, boundary, start)
+
+        try:
+            text = obj.queryText()
+        except NotImplementedError:
+            pass
+        except AttributeError:
+            debug.println(debug.LEVEL_FINE, "INFER _getLineContents: no obj")
+        else:
+            if start == None:
+                start = max(0, text.caretOffset)
+            else:
+                start = max(0, start - 1)
+            string, start, end = text.getTextAtOffset(start, boundary)
+            if string.endswith('\n'):
+                start = end
+
+        rv = self._script.utilities.getObjectsFromEOCs(obj, start, boundary)
         self._lineCache[key] = rv
 
         return rv
@@ -283,9 +316,13 @@ class LabelInference:
         except IndexError:
             index = len(contents)
 
-        onLeft = contents[max(0, index-1):index]
-        onLeft = [o for o in onLeft if o[0] and not self._isWidget(o[0])]
-        if not onLeft:
+        onLeft = contents[0:index]
+        for i in range(len(onLeft) - 1, -1, -1):
+            if self._isWidget(onLeft[i][0]):
+                onLeft = onLeft[(i+1):]
+                break
+
+        if not (onLeft and onLeft[0]):
             return None
 
         lObj, start, end, string = onLeft[-1]
@@ -295,8 +332,9 @@ class LabelInference:
 
         lExtents = self._getExtents(lObj, start, end)
         distance = extents[0] - (lExtents[0] + lExtents[2])
-        if distance <= proximity:
-            return string
+        if 0 <= distance <= proximity:
+            strings = [content[3] or content[0].name for content in onLeft]
+            return ''.join(strings)
 
         return None
 
@@ -312,6 +350,9 @@ class LabelInference:
         Returns the text which we think is the label, or None.
         """
 
+        if self._preventRight(obj):
+            return None
+
         extents = self._getExtents(obj)
         contents = self._getLineContents(obj)
         content = [o for o in contents if o[0] == obj]
@@ -321,8 +362,14 @@ class LabelInference:
             index = len(contents)
 
         onRight = contents[min(len(contents), index+1):]
-        onRight = [o for o in onRight if o[0] and not self._isWidget(o[0])]
-        if not onRight:
+        for i, item in enumerate(onRight):
+            if self._isWidget(item[0]):
+                if not self._preferRight(obj):
+                    return None
+                onRight = onRight[0:i]
+                break
+
+        if not (onRight and onRight[0]):
             return None
 
         rObj, start, end, string = onRight[0]
@@ -333,7 +380,8 @@ class LabelInference:
         rExtents = self._getExtents(rObj, start, end)
         distance = rExtents[0] - (extents[0] + extents[2])
         if distance <= proximity or self._preferRight(obj):
-            return string
+            strings = [content[3] or content[0].name for content in onRight]
+            return ''.join(strings)
 
         return None
 
@@ -350,6 +398,9 @@ class LabelInference:
         """
 
         thisLine = self._getLineContents(obj)
+        if not thisLine and thisLine[0]:
+            return None
+
         prevObj, start, end, string = thisLine[0]
         if obj == prevObj:
             start, end = self._script.utilities.getHyperlinkRange(prevObj)
@@ -379,7 +430,10 @@ class LabelInference:
             distance = objY - (y + height)
             if distance > proximity:
                 return None
-            if distance < 1:
+            if prevObj.getRole() == pyatspi.ROLE_TABLE_CELL \
+               and not prevObj in [obj.parent, obj.parent.parent]:
+                return None
+            if distance < 0:
                 continue
             if x + 150 < objX:
                 continue
@@ -401,28 +455,33 @@ class LabelInference:
         Returns the text which we think is the label, or None.
         """
 
-        thisLine = self._getLineContents(obj)
-        nextObj, start, end, string = thisLine[-1]
-        if obj == nextObj:
-            start, end = self._script.utilities.getHyperlinkRange(nextObj)
-            nextObj = nextObj.parent
-
-        try:
-            text = nextObj.queryText()
-        except (AttributeError, NotImplementedError):
+        if self._preventBelow(obj):
             return None
 
+        thisLine = self._getLineContents(obj)
+        if not thisLine and thisLine[0]:
+            return None
+
+        lastObj, start, end, string = thisLine[-1]
+        if obj == lastObj:
+            start, end = self._script.utilities.getHyperlinkRange(obj)
+            lastObj = lastObj.parent
+
         objX, objY, objWidth, objHeight = self._getExtents(obj)
         if not (objWidth and objHeight):
             return None
 
         boundary = pyatspi.TEXT_BOUNDARY_LINE_START
-        line = text.getTextAfterOffset(end - 1, boundary)
-        string = line[0].strip()
-        if string:
+        nextLine = self._script.utilities.getObjectsFromEOCs(
+            lastObj, end + 1, boundary)
+        if not (nextLine and nextLine[0]):
+            return None
+
+        nextObj, start, end, string = nextLine[0]
+        if string.strip():
             x, y, width, height = self._getExtents(nextObj, start, end)
             distance = y - (objY + objHeight)
-            if distance <= proximity:
+            if 0 <= distance <= proximity:
                 return string
 
         return None
@@ -444,6 +503,9 @@ class LabelInference:
         if not self._isSimpleObject(cell):
             return None
 
+        if not cell in [obj.parent, obj.parent.parent]:
+            return None
+
         pred = lambda x: x.getRole() == pyatspi.ROLE_TABLE
         grid = pyatspi.utils.findAncestor(cell, pred)
         if not grid:
@@ -464,23 +526,37 @@ class LabelInference:
             if label:
                 return label
 
-        if col < table.nColumns:
+        if col < table.nColumns and not self._preventRight(obj):
             candidate = table.getAccessibleAt(row, col + 1)
             label = self._createLabelFromContents(candidate)
             if label:
                 return label
 
+        cellAbove = cellBelow = labelAbove = labelBelow = None
         if row > 0:
-            candidate = table.getAccessibleAt(row - 1, col)
-            label = self._createLabelFromContents(candidate)
-            if label:
-                return label
+            cellAbove = table.getAccessibleAt(row - 1, col)
+            labelAbove = self._createLabelFromContents(cellAbove)
+            if labelAbove and self._preferTop(obj):
+                return labelAbove
 
-        if row < table.nRows and not self._preferTop(obj):
-            candidate = table.getAccessibleAt(row + 1, col)
-            label = self._createLabelFromContents(candidate)
-            if label:
-                return label
+        if row < table.nRows and not self._preventBelow(obj):
+            cellBelow = table.getAccessibleAt(row + 1, col)
+            labelBelow = self._createLabelFromContents(cellBelow)
+
+        if labelAbove and labelBelow:
+            objX, objY, objWidth, objHeight = self._getExtents(obj)
+            aboveX, aboveY, aboveWidth, aboveHeight = self._getExtents(cellAbove)
+            belowX, belowY, belowWidth, belowHeight = self._getExtents(cellBelow)
+            dAbove = objY - (aboveY + aboveHeight)
+            dBelow = belowY - (objY + objHeight)
+            if dAbove <= dBelow:
+                return labelAbove
+            return labelBelow
+
+        if labelAbove:
+            return labelAbove
+        if labelBelow:
+            return labelBelow
 
         # None of the cells immediately surrounding this cell seem to be serving
         # as a functional label. Therefore, see if this table looks like a grid
@@ -490,10 +566,8 @@ class LabelInference:
             return None
 
         cells = [table.getAccessibleAt(i, col) for i in range(1, table.nRows)]
-        if [x for x in cells if x == None]:
-            debug.println(debug.LEVEL_FINE, "INFER: Potentially broken table!")
-            return None
-        if [x for x in cells if x[0] and x[0].getRole() != obj.getRole()]:
+        cells = [x for x in cells if x != None]
+        if [x for x in cells if x.childCount and x[0].getRole() != obj.getRole()]:
             return None
 
         label = self._createLabelFromContents(firstRow[col])
