@@ -260,6 +260,11 @@ class SpeechGenerator(generator.Generator):
         role = args.get('role', obj.getRole())
 
         doNotPresent = [pyatspi.ROLE_UNKNOWN]
+
+        # egg-list-box, e.g. privacy panel in gnome-control-center
+        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+            doNotPresent.append(pyatspi.ROLE_FILLER)
+
         if _settingsManager.getSetting('speechVerbosityLevel') \
                 == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
